<?php

	define('APS_DEVELOPMENT_MODE', true);
	require "aps/2/runtime.php";	

	class CPU {
		/**
		 * @type("integer")
		 * @title("Number of CPUs")
		 * @description("Number of CPU cores")
		 */
		public $number;
	}
	
	class OS {
		/**
		 * @type("string")
		 * @title("OS Name")
		 * @description("Operating System Name")
		 */
		public $name;
	
		/**
		 * @type("string")
		 * @title("OS Version")
		 * @description("Operating System version")
		 */
		public $version;
	}
	
	class Hardware {
		/**
		 * @type("integer")
		 * @title("RAM Size")
		 * @description("RAM size in GB")
		 */
		public $memory;
	
		/**
		 * @type("integer")
		 * @title("Disk Space")
		 * @description("Disk space in GB")
		 */
		public $diskspace;
	
		/**
		 * @type("CPU")
		 * @title("CPU")
		 * @description("Server CPU parameters")
		 */
		public $CPU;
		
		/**
		 * @type("boolean")
		 * @title("VM with separate OS kernel or Container using shared OS kernel")
		 * @description("VM requires more time for provisioning - async POST")
		 */
		public $VM;
	}
	
	class Platform {
		/**
		 * @type("string")
		 * @title("Architecture")
		 * @description("Platform architecture")
		 */
		public $arch;
	
		/**
		 * @type("OS")
		 * @title("OS Parameters")
		 * @description("Parameters of operating system")
		 */
		public $OS;
	}
	
	/**
	* @type("http://aps-standard.org/samples/async1pn/vps/1.0")
	* @implements("http://aps-standard.org/types/core/resource/1.0")
	*/
	class vps extends \APS\ResourceBase
	{

		/**
		 * @link("http://aps-standard.org/samples/async1pn/context/1.0")
		 * @required
		*/
		public $context;

		/**
		 * @type("string")
		 * @title("name")
		 * @description("Server Name")
		 */
		public $name;
		
		/**
		 * @type("string")
		 * @title("Description")
		 * @description("Server Description")
		 */
		public $description;
		
		/**
		 * @type("string")
		 * @title("state")
		 * @description("Server State")
		 */
		public $state;
		
		/**
		 * @type("Hardware")
		 * @title("Hardware")
		 * @description("Server Hardware")
		 */
		public $hardware;
		
		/**
		 * @type("Platform")
		 * @title("Platform")
		 * @description("OS Platform")
		 */
		public $platform;
		
		/**
		 * @type("integer")
		 * @title("Counter for async operation")
		 * @description("Counts the number of retries for async operation")
		 */
		public $retry;

		/**
		 * @type(string)
		 * @title("Notification ID")
		 */
		public $notificationId;
				
		// Provision method is called when a sync provisioning is requested
		public function provision() {
			
			// Get an instance of the Notification Manager:
			$notificationManager = \APS\NotificationManager::getInstance();
			
			// Get ready account APS ID needed for the notification structure.
			$accountId = $this->context->account->aps->id;
						
			$notification = new \APS\Notification;
			$notification->message = new \APS\NotificationMessage("VPS Creation");
			$notification->details = new \APS\NotificationMessage(
			    "VPS __name__ is provisioned",
			    array("name" => $this->name)
			);
			$notification->accountId = $accountId;
			$notification->status = \APS\Notification::ACTIVITY_READY;
			$notification->packageId = $this->aps->package->id;
			$notification->link = new \APS\NotificationLink(
			    "/v/aps/samples/async1pn/servers",                      // Main link
			    "/v/aps/samples/async1pn/server-edit/r/".$this->aps->id, // Link more
			    "View or edit VPS"                                       // Link more text
			);

			
			// VM requires more time, thus the *async* operation is required
			if ($this->hardware->VM) {
				$this->state = "Creating"; // VM state is intermediate
			
				// Edit the notification message
				$notification->message->message = "VM Creation";
				$notification->details->message = "Provisioning of VM __name__ is started";
				$notification->status = \APS\Notification::ACTIVITY_IN_PROGRESS;
				$notification->link->linkMore = "/v/aps/samples/async1pn/servers";
				$notification->link->linkMoreText->message = "View Servers";
			}
				
			// Send the notification
			$notificationResponse = $notificationManager->sendNotification($notification);
			
			// Store the notification ID as the VPS property to update or remove it in other operations
			$this->notificationId = $notificationResponse->id;
			
			$this->retry = 0;
			
			if ($this->hardware->VM) {
				throw new \Rest\Accepted($this, "Creating VPS", 5); // Return "202 Accepted"
			}
			else $this->retry = 5;
		}

		public function provisionAsync() {
			// Get an instance of the Notification Manager:
			$notificationManager = \APS\NotificationManager::getInstance();
			
			// Get ready account APS ID needed for the notification structure.
			$accountId = $this->context->account->aps->id;
			
			// Increment the retry counter
			$this->retry +=1;

			$notification = new \APS\Notification;
			$notification->message = new \APS\NotificationMessage("VM Creation");
			$notification->details = new \APS\NotificationMessage(
					"Provisioning of VM __name__ is in progress",
					array("name" => $this->name)
			);
			$notification->fullDetails = new \APS\NotificationMessage(
					"__percent__ % of the Virtual Machine provisioning is completed",
					array("percent" => $this->retry * 20)
			);
			$notification->link = new \APS\NotificationLink(
					"",
					"/v/aps/samples/async1pn/servers",
					"View Servers"
			);
			
			if ($this->retry >= 5) {
			   $this->state = "Stopped";  // Finish the Async operation - return 200 OK to APSC
			   $notification->details->message = "Provisioning of VM __name__ is completed";
			   $notification->status = \APS\Notification::ACTIVITY_READY;
			   $notification->link->linkMore="/v/aps/samples/async1pn/server-edit/r/".$this->aps->id;
			   $notification->link->linkMoreText->message = "View or edit VM";
			}
			
			if (isset($this->notificationId)) {
				$notificationResponce = $notificationManager->editNotification(
						$this->notificationId,
						$notification
				);}
				
			if ($this->retry < 5) {
			   // Require the next async retry
			   throw new \Rest\Accepted($this, "Creating VM", 15); // Return code "202 Accepted"
			}

		}
		
		public function unprovision() {
			
			$accountId = $this->context->account->aps->id;
			
			// Get an instance of the Notification Manager:
			$notificationManager = \APS\NotificationManager::getInstance();
			
			$notification = new \APS\Notification;
			$notification->message = new \APS\NotificationMessage("Server Removal");
			$notification->details = new \APS\NotificationMessage(
					"Server __name__ is removed",
					array("name" => $this->name)
			);
			$notification->type = \APS\Notification::TYPE_WARNING;
			$notification->accountId = $accountId;
			$notification->status = \APS\Notification::WARNING_UPCOMING;
			$notification->packageId = $this->aps->package->id;
			$notification->link = new \APS\NotificationLink(
					"/v/aps/samples/async1pn/servers",
					"/v/aps/samples/async1pn/servers",
					"View Servers"
			);
			
			// Get a list of VPSEs to count the remained ones
			$apsc = \APS\Request::getController();
			$vpses = $apsc->getResources(
					'implementing(http://aps-standard.org/samples/async1pn/vps/1.0)'
			);
			
			// Removing the last server of the customer?
			if (count($vpses) <= 1) {
				$notification->details->message = "The last server __name__ is removed";
				$notification->status = \APS\Notification::WARNING_EXISTING;
			}
			
			// If Notification Manager is available, remove the notification
			if (isset($this->notificationId)) {
			
				// Remove the stored Notification
				$notificationManager->deleteNotification($this->notificationId);
				unset($this->notificationId);
			
				// Send the new notification
				$notificationManager->sendNotification($notification);
			}
		}
		
		/**
		 * @verb(GET)
		 * @path("/start")
		 * @return(string,text/json)
		 */
		public function start() {
			$this->state = 'Running';
			$apsc = \APS\Request::getController();
			$apsc->updateResource($this);
		}
		
		/**
		 * @verb(GET)
		 * @path("/stop")
		 * @return(string,text/json)
		 */
		public function stop() {
			$this->state = 'Stopped';
			$apsc = \APS\Request::getController();
			$apsc->updateResource($this);
		}
	}

?>
